#! /usr/bin/env python
#
#     Copyright (C) 2005 Ronan Keegan
#
#     This code is distributed under the terms and conditions of the
#     CCP4 Program Suite Licence Agreement as a CCP4 Application.
#     A copy of the CCP4 licence can be obtained by writing to the
#     CCP4 Secretary, Daresbury Laboratory, Warrington WA4 4AD, UK.
#
#
# Python script to handle PDB files.
# Ronan Keegan 15/11/04

import os, sys, string

class PDB_info:
  """ This classs is used to extract any relavent information from a PDB file. """

  def __init__(self):
     self.residues=[]
     self.expdata=''
     self.SEQRESsequence=''
     self.ATOMsequence=''
     self.sequence_length=0
     self.residue_map={'ALA' : 'A', 'CYS' : 'C', 'ASP' : 'D', 'GLU' : 'E',
          'PHE' : 'F', 'GLY' : 'G', 'HIS' : 'H', 'ILE' : 'I', 'LYS' : 'K', 
          'LEU' : 'L', 'MET' : 'M', 'ASN' : 'N', 'PRO' : 'P', 'GLN' : 'Q', 
          'ARG' : 'R', 'SER' : 'S', 'THR' : 'T', 'VAL' : 'V', 'TRP' : 'W', 
          'TYR' : 'Y', 'ADE' : 'A', 'CYT' : 'C', 'GUA' : 'G', 'THY' : 'T',
          'MSE' : 'M', 'TTQ' : 'W', 'DAL' : 'A'}
     try:
        self.debug=eval(os.environ['MRBUMP_DEBUG'])
     except:
        self.debug=False

  def setDEBUG(self, flag):
     self.debug=flag 

  def getSEQRES(self, pdb_name, chain):
     """ Read the SEQRES record from a PDB file and return its sequence in single
     letter format for a given chain id. If no chain id in file, assumes chain 
     A. """  

     PDBfile=open(pdb_name,'r')
     line=PDBfile.readline()
     raw_sequence=[]
     unknown_res=0

     while line:
        # Extract sequence from SEQRES records
        if 'SEQRES' in line[0:7]:
           list=(string.split(line[19:70]))
           chain_id = line[11]
           if chain_id == ' ':
              chain_id = 'A'
           if chain_id == chain:
              for res in list:
                 self.residues.append(res)
                 if self.residue_map.has_key(res) == False:
                    raw_sequence.append('X')
                    unknown_res = unknown_res + 1
                 else:
                    raw_sequence.append(self.residue_map[res])
        line=PDBfile.readline()
     PDBfile.close()
     sequence=string.join(raw_sequence).replace(' ','')
     self.sequence_length=len(sequence)
   
     self.SEQRESsequence = sequence
     return sequence
  
  def getSequence(self, pdb_name, chain=''):
     """ Read the ATOM records from a PDB file and return its sequence in single
     letter format for a given chain ID. If no chain id present in PDB file it 
     will assume chain A. If no value for chain is given in input, the entire 
     sequence is extracted from the PDB. If the file contains models it will 
     extract the sequence from the first model. Returns: string"""  

     PDBfile=open(pdb_name,'r')
     line=PDBfile.readline()
     raw_sequence=[]
     unknown_res=0
     prev_res_number=0
     model_no=1

     while line:
        # Test for pressence of models.
        if 'MODEL' in line[0:6]:   
           model_no = int(string.strip(line[11:14]))
        if model_no != 1:
           break
        # Retrieve atom record.
        if 'ATOM' in line[0:6]:
           res_number = line[23:26]
           chain_id = line[21]
           if chain_id == ' ':
              chain_id = 'A'
           if res_number != prev_res_number and (chain_id == chain or chain == ''):
              res = line[17:20]
              if self.residue_map.has_key(res) == False:
                 raw_sequence.append('X')
                 unknown_res = unknown_res + 1
              else:
                 raw_sequence.append(self.residue_map[res])
              prev_res_number = res_number
        line=PDBfile.readline()
     PDBfile.close()
  
     sequence=string.join(raw_sequence).replace(' ','')
     self.sequence_length=len(sequence)

     self.ATOMsequence = sequence
     return sequence     

  def getEXPDTA(self, pdb_name):
     """ Make a record of the EXPDTA (experimental data) section in the PDB file \
     for later processing. Useful for excluding NMR models."""

     PDBfile=open(pdb_name,'r')
     line=PDBfile.readline()

     while line:
        # Look for the EXPDTA line 
        if 'EXPDTA' in line[0:7]:
          self.expdata = self.expdata + " " + string.strip(line)[9:]
        line=PDBfile.readline()

     PDBfile.close()
   
     return self.expdata

