#! /usr/bin/env python
#
#     Copyright (C) 2005 Ronan Keegan
#
#     This code is distributed under the terms and conditions of the
#     CCP4 Program Suite Licence Agreement as a CCP4 Application.
#     A copy of the CCP4 licence can be obtained by writing to the
#     CCP4 Secretary, Daresbury Laboratory, Warrington WA4 4AD, UK.
#
#
#
# Setup Ensmeble for Phaser MR
# Ronan Keegan 12/12/04 

import os, sys, string
import shutil

if not os.environ.has_key('CCP4'):
    raise RuntimeError, 'CCP4 must be defined'

import Model_struct

class Ensemble:
   """ A class to construct an ensemble of the top match PDBs for Phaser. """

   def __init__(self):
      self.superposeEXE='superpose' 
      self.pdbsetEXE='pdbset'
 
      self.SPxyz_template=''
      self.SPxyzin=''
      self.SPxyzout=''

      self.SP_logfile='' 

      self.number_ensem=0
      try:
         self.debug=eval(os.environ['MRBUMP_DEBUG'])
      except:
         self.debug=False

   def setDEBUG(self, flag):
      self.debug=flag

   def setSPXYZ_TEMPLATE(self, xyzin):
      self.SPxyz_template=xyzin

   def setSPXYZIN(self, xyzin):
      self.SPxyzin=xyzin

   def setSPXYZOUT(self, xyzout):
      self.SPxyzout=xyzout

   def setSPLOGFILE(self, filename):
      self.SP_logfile=filename

   def setNumEnsem(self, number_ensem):
      self.number_ensem=number_ensem

   def setup_ensemble(self, mstat, ENSEMNUM):
      """  A function to setup structure data for running Phaser with ensembles. Calls superpose
      to line up each of the template structures with each other. """

      count=0

      # Set up the model list entry
      m=Model_struct.Models_struct()
      m.setNumberEnsem(ENSEMNUM)
      m.setModelChainSource('ensm__')
      m.setPDBName('ensm')
      m.setModelName('ensm__')
      m.setModelType('ENSMBL')
      m.setModel_directory(mstat.search_dir + '/data/ensm__')

      # Create the directory structure
      os.mkdir(m.model_directory) 
      os.mkdir(m.model_directory + "/mr") 
      os.mkdir(m.model_directory + "/refine") 
      os.mkdir(m.model_directory + "/mr/pdbfiles") 
      os.mkdir(m.model_directory + "/mr/logs") 
      os.mkdir(m.model_directory + "/mr/tmp") 

      self.setSPLOGFILE(m.model_directory + '/mr/logs/superpose.log')

      print "Number of seach models in Ensemble: %d" % ENSEMNUM
      print "Using Superpose to line models up. Superpose log:"
      print m.model_directory + '/mr/logs/superpose.log'
      print ""

      for chain in mstat.sorted_MR_list:

         # We only want to include monomers and domains in the ensemble, ignore multimers
         if chain[5] not in ['0','1','2','3','4','5','6','7','8','9']:
         # First setup the chain details. If there is no chain for a pdb we just 
         # set the chain details to be a blank space in the superpose command line.

         # Top match entry will be our base alignment.  
            if count == 0:

               print "Base alignment model = %s" % chain

               # Set the template structure for the ensemble superposition
               self.setSPXYZ_TEMPLATE(mstat.chain_list[chain].PDBFile)
               self.setSPXYZOUT(m.model_directory + '/mr/pdbfiles/' + mstat.chain_list[chain].chainName + '.pdb')
               shutil.copyfile(self.SPxyz_template, self.SPxyzout)

            # All other entries will be aligned to top match 
            else:               

               print "Alignment model %d is %s" % (count, chain)

               self.setSPXYZIN(mstat.chain_list[chain].PDBFile)
               self.setSPXYZOUT(m.model_directory + '/mr/pdbfiles/' + mstat.chain_list[chain].chainName + '.pdb')

               o=os.popen(self.superposeEXE + " "  + self.SPxyzin  + " " + self.SPxyz_template + " " + self.SPxyzout \
                   + " >> " + self.SP_logfile, 'r')
               o.close()

            # Add this pdb code to the ensemble list (along with its seq ID)

            m.PDBfile.append(self.SPxyzout)
            m.seqID.append(mstat.chain_list[chain].seqID)
         
            # When we reach the number of ensemble structures to be used, break out.
            count=count+1
            if count >= m.number_ensem:
               print ""
               break 
    
      if count < m.number_ensem:
         m.number_ensem = count

      # Add the ensemble to the model list      
      mstat.model_list[m.name]=m
       
      # Insert the ensemble at the top of the sorted list
      mstat.sorted_MR_list.insert(0,m.name)
          
