#! /usr/bin/env python
#
#     Copyright (C) 2005 Ronan Keegan
#
#     This code is distributed under the terms and conditions of the
#     CCP4 Program Suite Licence Agreement as a CCP4 Application.
#     A copy of the CCP4 licence can be obtained by writing to the
#     CCP4 Secretary, Daresbury Laboratory, Warrington WA4 4AD, UK.
#
# An MrBUMP wrapper for running Molrep jobs
# Ronan Keegan - 8/03/06
#

import os, sys, string
import CCP4_Submit

class Molrep:
   """ A class to submit Molrep jobs to the local machine or a batch system on a cluster. """

   def __init__(self):
      self.logfile=""

   def setLogfile(self, filename):
      self.logfile=filename

   def submit_job(self, init, mstat, model, target_info):
      """ Function to launch a molrep/refmac5 job for a given model. """

      # Create a job and first set up the Molrep details

      # Check to see if the logfile has been specified, otherwise give it a default name
      if self.logfile=="":
         self.logfile=model.model_directory + "/molrep_refmac_job.log" 

      # Setup the job
      MR_job=CCP4_Submit.CCP4_sub()
      if os.name=="nt":
         MR_job.setProgram("molrep.exe")
      else:
         MR_job.setProgram("molrep")
      MR_job.setProgramLogfile(model.molrep_logfile)
      MR_job.setCommandLine("HKLIN %s MODEL %s" % \
                            (target_info.hklin,
                             model.PDBfile[0]))

      # Setup the keywords for the job
      MR_job.setKEYWORD('DOC Y') 
      MR_job.setKEYWORD('LABIn F=' + target_info.mtz_coldata['F'] \
                           + ' SIGF=' + target_info.mtz_coldata['SIGF'])
      MR_job.setKEYWORD('NMON ' + `target_info.no_of_mols/model.number_mols`) 
      MR_job.setKEYWORD('SIM %.3f' % model.seqID[0]) 
      MR_job.setKEYWORD('END')

      # Add a job command to move the align.pdb file to a job related name
      if os.name == "nt":
         MR_job.addProgram("move")
         MR_job.setProgramLogfile("move.log")
         MR_job.setCommandLine("molrep.pdb " + model.molrep_PDBfile) 
      else:
         MR_job.addProgram("mv")
         MR_job.setProgramLogfile("mv.log")
         MR_job.setCommandLine(model.model_directory + "/mr/molrep.pdb " + model.molrep_PDBfile) 

      # Add the refinement with Refmac job details
      MR_job.addProgram("refmac5")
      MR_job.setProgramLogfile(model.refmac_logfile)
      MR_job.setCommandLine("HKLIN %s HKLOUT %s XYZIN %s XYZOUT %s" %
                            (model.refmac_MTZINfile, \
                             model.refmac_MTZOUTfile, \
                             model.molrep_PDBfile, \
                             model.refmac_PDBfile))

      MR_job.setKEYWORD('LABIn FP=' + target_info.mtz_coldata['F'] \
                           + ' SIGFP=' + target_info.mtz_coldata['SIGF'] \
                           + ' FREE=' + target_info.mtz_coldata['FREE'])
      MR_job.setKEYWORD('NCYC %d' % init.keywords.NCYC)
      MR_job.setKEYWORD('MAKE HYDR N')
      MR_job.setKEYWORD('WEIGHT AUTO')
      MR_job.setKEYWORD('END')

      # Fire off the job

      if init.keywords.CLUSTER:
         # Submit the job to a batch queue
         MR_job.submit_job(model.molrep_jobname,\
                           self.logfile,\
                           model.model_directory + "/mr",\
                           "SGE")
         # Note the job number and the job name
         model.setMR_JobID(MR_job.job_number)
         mstat.jobid_array.add(model.MR_jobID)
         mstat.jobid_dict[model.MR_jobID]=model.molrep_jobname
         mstat.jobname_dict[model.molrep_jobname]=model.name
      else:
         # Run the job locally
         MR_job.submit_job(model.molrep_jobname,\
                           self.logfile,\
                           model.model_directory + "/mr")


