#! /usr/bin/env python
#
#     Copyright (C) 2005 Ronan Keegan
#
#     This code is distributed under the terms and conditions of the
#     CCP4 Program Suite Licence Agreement as a CCP4 Application.
#     A copy of the CCP4 licence can be obtained by writing to the
#     CCP4 Secretary, Daresbury Laboratory, Warrington WA4 4AD, UK.
#
# An MrBUMP wrapper for running Phaser jobs
# Ronan Keegan - 8/03/06
#

import os, sys, string
import CCP4_Submit

class Phaser:
   """ A class to submit Phaser jobs to the local machine or a batch system on a cluster. """

   def __init__(self):
      self.logfile=""

   def setLogfile(self, filename):
      self.logfile=filename

   def submit_job(self, init, mstat, model, target_info):
      """ Function to launch a phaser/refmac5 job for a given model. """

      # Create a job and first set up the Phaser details

      # Check to see if the logfile has been specified, otherwise give it a default name
      if self.logfile=="":
         self.logfile=model.model_directory + "/phaser_refmac_job.log" 

      # Setup the job
      MR_job=CCP4_Submit.CCP4_sub()
      MR_job.setProgram("phaser")
      MR_job.setProgramLogfile(model.phaser_logfile)
      MR_job.setCommandLine("")

      # Setup the keywords for the job
      MR_job.setKEYWORD("MODE MR_AUTO")
      MR_job.setKEYWORD("HKLIn %s" % target_info.hklin)
      MR_job.setKEYWORD("LABIn F=" + target_info.mtz_coldata['F'] + " SIGF=" + target_info.mtz_coldata['SIGF'])
      MR_job.setKEYWORD("COMPosition PROTein MW %.3f NUM %d" % (target_info.mol_weight, target_info.no_of_mols))
        
      # Set up the ensemble details
      ensm_line = 'ENSEmble autoMR '
      for i in range(model.number_ensem):
         ensm_line += 'PDBFile ' + model.PDBfile[i] + ' IDENtity %.3f ' % model.seqID[i]

      MR_job.setKEYWORD(ensm_line)
      MR_job.setKEYWORD("SEARCH ENSEmble autoMR NUM " + `target_info.no_of_mols/model.number_mols`)
      MR_job.setKEYWORD("XYZOUT ON")
      MR_job.setKEYWORD("PACK %d" % init.keywords.PACK)
      MR_job.setKEYWORD("ROOT " + model.model_directory + '/mr/phaser_' + model.name)
      MR_job.setKEYWORD("END")

      # Add the refinement with Refmac job details
      MR_job.addProgram("refmac5")
      MR_job.setProgramLogfile(model.refmac_logfile)
      MR_job.setCommandLine("HKLIN %s HKLOUT %s XYZIN %s XYZOUT %s" %
                            (model.refmac_MTZINfile, \
                             model.refmac_MTZOUTfile, \
                             model.phaser_PDBfile, \
                             model.refmac_PDBfile))

      MR_job.setKEYWORD('LABIn FP=' + target_info.mtz_coldata['F'] \
                           + ' SIGFP=' + target_info.mtz_coldata['SIGF'] \
                           + ' FREE=' + target_info.mtz_coldata['FREE'])
      MR_job.setKEYWORD('NCYC %d' % init.keywords.NCYC)
      MR_job.setKEYWORD('MAKE HYDR N')
      MR_job.setKEYWORD('WEIGHT AUTO')

      # Fire off the job

      if init.keywords.CLUSTER:
         # Submit the job to a batch queue
         MR_job.submit_job(model.phaser_jobname,\
                           self.logfile,\
                           model.model_directory + "/mr",\
                           "SGE")
         # Note the job number and the job name
         model.setMR_JobID(MR_job.job_number)
         mstat.jobid_array.add(model.MR_jobID)
         mstat.jobid_dict[model.MR_jobID]=model.phaser_jobname
         mstat.jobname_dict[model.phaser_jobname]=model.name
      else:
         # Run the job locally
         MR_job.submit_job(model.phaser_jobname,\
                           self.logfile,\
                           model.model_directory + "/mr")


