#! /usr/bin/env python
#
#     Copyright (C) 2005 Ronan Keegan
#
#     This code is distributed under the terms and conditions of the
#     CCP4 Program Suite Licence Agreement as a CCP4 Application.
#     A copy of the CCP4 licence can be obtained by writing to the
#     CCP4 Secretary, Daresbury Laboratory, Warrington WA4 4AD, UK.
#
#
#
# Wrapper for Mafft the multiple alignment program 
# Ronan Keegan 10/9/05

import os, sys, string


class Mafft:
   """ A class to call Mafft, the sequence multiple alignment program, to do 
       alignmnet of target and template sequences."""
 
   def __init__(self):

      self.mafft_EXE = 'mafft'

      self.input_fasta_file = ''
      self.output_fasta_file = ''
      self.log_file = ''
      try:
         self.debug=eval(os.environ['MRBUMP_DEBUG'])
      except:
         self.debug=False

   def setDEBUG(self, flag):
      self.debug=flag  

   def setInputFastaFile(self, file_name):
      self.input_fasta_file = file_name

   def setOutputFastaFile(self, file_name):
      self.output_fasta_file = file_name

   def setLogFile(self, file_name):
      self.log_file = file_name

   def Mafft(self, input_fasta_file, output_fasta_file):
      """ A function to do a system call to Mafft."""

      self.setInputFastaFile(input_fasta_file)
      self.setOutputFastaFile(output_fasta_file)
      self.setLogFile('mafft.log')
        
      if self.debug:
         print ""
         print "======================"
         print "Mafft command line:"
         print "======================"
         print self.mafft_EXE + ' --quiet --maxiterate 1000 --nofft ' + self.input_fasta_file + \
                ' > ' + self.output_fasta_file 
         print ""

      os.system(self.mafft_EXE + ' --quiet --maxiterate 1000 --nofft ' + self.input_fasta_file + \
          ' > ' + self.output_fasta_file)
  
   def get_raw_target_len(self):
      """ A function to get the length of the target sequence before it is processed by the alignment program. """

      seq = ''
 
      file = open(self.input_fasta_file, 'r')

      line = file.readline()
      name=string.strip(line).replace('>','') 
      line = file.readline()
      while line:
         if '>' in line:
            break
         else:
           seq = seq + string.strip(line)
         line = file.readline()
    
      return len(seq)
   
   def convert_to_pir(self, align_file):
      """ Convert a fasta formated alignment file to PIR format (for Chainsaw). """
    
      # Setup temporary variables

      target_name = ''
      target_seq = ''
      templt_name = ''
      templt_seq = ''

      # Open the fasta file to read the target & template details

      in_file=open(align_file,'r')
      
      line = in_file.readline()
      target_name = string.strip(line).replace('>','')
      line = in_file.readline()
      while line:
         if '>' not in line:
            target_seq = target_seq + string.strip(line)
         else:
            break
         line = in_file.readline() 
 
      templt_name = string.strip(line).replace('>','')
      line = in_file.readline() 
      while line:
         templt_seq = templt_seq + string.strip(line)
         line = in_file.readline() 
         
      in_file.close()

      # Re-open the file and write the details in PIR format

      out_file=open(align_file, 'w')
       
      out_file.write(">P1;" + target_name + "\n")
      out_file.write("\n")
      out_file.write(target_seq + "\n")
      out_file.write("*\n")
      out_file.write(">P1;" + templt_name + "\n")
      out_file.write("\n")
      out_file.write(templt_seq + "\n")
      out_file.write("*\n")

      out_file.close()

         
           
if __name__ == '__main__':

   mafft=Mafft()
   mafft.Mafft('1vrd_all.fasta', 'out_1vrd.fasta')
