#! /usr/bin/env python
#
#     Copyright (C) 2005 Ronan Keegan 
#
#     This code is distributed under the terms and conditions of the
#     CCP4 Program Suite Licence Agreement as a CCP4 Application.
#     A copy of the CCP4 licence can be obtained by writing to the
#     CCP4 Secretary, Daresbury Laboratory, Warrington WA4 4AD, UK.
#

import os, string, sys
import shutil
import PDB_info
import popen2

import urllib
import sets
#from sets import Set

import Align_structure
import Write_seq_align
import Align_score 
import Sort_dict
import Get_sequence_DB
import Fasta_search
import SCOP_search, Get_SCOP
import Write_MR_results
import Write_Search_results

import MRBUMP_MAFFT
import MRBUMP_CLUSTALW

if not os.environ.has_key('CCP4'):
    raise RuntimeError, 'CCP4 not found'
if not os.environ.has_key('CBIN'):
    raise RuntimeError, 'CBIN not found'


class Chain_struct:
   """ A defined structure for the matching sequence
   information. """
	
   def __init__(self):
     self.chainName=''
     self.chainID=''
     self.PDBName=''
     self.unprocessed_PDBFile=''
     self.PDBFile=''
     self.Score=0
     self.seqID=0.0
     self.align_length=0
     self.sequence=''
     self.chain_sequence=''
     self.seqlength=0

     self.no_of_domains=0
     self.domain_list=[]
     self.domain_range_list=[]
     self.no_of_ranges=0

     self.source=''
     self.parent_chain=''
     self.expdata=''
 
     self.working_DIR=''
 
     self.raw_align_file=''
     self.ma_pir_file='' 

     self.plyala_modelPDB='' 
     self.plyala_logfile='' 
     self.plyala_jobname=''
     self.plyala_jobid=''
     self.plyala_score=0.0

     self.molrep_modelPDB='' 
     self.molrep_logfile='' 
     self.molrep_jobname=''
     self.molrep_jobid=''
     self.molrep_score=0.0

     self.pdbclip_modelPDB='' 
     self.coord_format_logfile=''
     self.pdbset_logfile=''
     self.pdbcur_logfile=''

     self.chainsaw_modelPDB='' 
     self.chainsaw_logfile='' 
     self.chainsaw_jobname=''
     self.chainsaw_jobid=''
     self.chainsaw_score=0.0

     self.multimer_type=''
     self.has_multimer='no'
     self.multimer_list=[]

     self.alignment=Align_structure.align_struct()

   def setchainName(self, chainName):
     self.chainName=chainName 

   def setchainID(self, chainID):
     self.chainID=chainID

   def setPDBName(self, PDBName):
     self.PDBName=PDBName 

   def setUnprocessedPDBFile(self, PDBFile):
     self.unprocessed_PDBFile=PDBFile 

   def setPDBFile(self, PDBFile):
     self.PDBFile=PDBFile 

   def setScore(self, Score):
     self.Score=Score 

   def setseqID(self, seqID):
     self.seqID=seqID 

   def setAlignLength(self, align_length):
     self.align_length=align_length 

   def setSequence(self, sequence):
     self.sequence=sequence 

   def setChainSequence(self, chain_sequence):
     self.chain_sequence=chain_sequence 

   def setSeqLength(self, seqlength):
     self.seqlength=seqlength 

   def setNoofDomains(self, number):
     self.no_of_domains=number

   def setNoofRanges(self, number):
     self.no_of_ranges=number

   def setSource(self, sourcetype):
     self.source=sourcetype 

   def setParentChain(self, name):
     self.parent_chain=name 

   def setEXPDATA(self, expdata):
     self.expdata=expdata 


   def setWorkingDIR(self, directory):
     self.working_DIR=directory


   def setRawAlignFile(self, raw_align_file):
      self.raw_align_file = raw_align_file

   def setMAPIRFile(self, filename):
     self.ma_pir_file=filename 


   def setPlyalaModelPDB(self, Plyala_model_PDB):
     self.plyala_modelPDB=Plyala_model_PDB 

   def setPlyalaLogFile(self, Plyala_Logfile):
     self.plyala_logfile=Plyala_Logfile

   def setPlyalaJobName(self, Plyala_Job_Name):
     self.plyala_jobname=Plyala_Job_Name 

   def setPlyala_jobid(self, Plyala_Job_ID):
     self.plyala_jobid=Plyala_Job_ID 

   def setPlyalaScore(self, PlyalaScore):
     self.plyala_score=PlyalaScore 


   def setMolrepModelPDB(self, Molrep_model_PDB):
     self.molrep_modelPDB=Molrep_model_PDB 

   def setMolrepLogFile(self, Molrep_Logfile):
     self.molrep_logfile=Molrep_Logfile

   def setMolrepJobName(self, Molrep_Job_Name):
     self.molrep_jobname=Molrep_Job_Name 

   def setMolrep_jobid(self, Molrep_Job_ID):
     self.molrep_jobid=Molrep_Job_ID 

   def setMolrepScore(self, MolrepScore):
     self.molrep_score=MolrepScore 


   def setClipModelPDB(self, Clip_model_PDB):
     self.pdbclip_modelPDB=Clip_model_PDB 

   def setCoordFormatLog(self, coord_format_logfile):
     self.coord_format_logfile=coord_format_logfile 

   def setPDBsetLog(self, pdbset_logfile):
     self.pdbset_logfile=pdbset_logfile 

   def setPDBcurLog(self, pdbcur_logfile):
     self.pdbcur_logfile=pdbcur_logfile 


   def setChainsawModelPDB(self, model_PDB):
     self.chainsaw_modelPDB=model_PDB 

   def setChainsawLogFile(self, logfile):
     self.chainsaw_logfile=logfile

   def setChainsawJobName(self, Job_Name):
     self.chainsaw_jobname=Job_Name 

   def setChainsaw_jobid(self, Job_ID):
     self.chainsaw_jobid=Job_ID 

   def setChainsawScore(self, Score):
     self.chainsaw_score=Score 


   def setMultimerType(self, type):
     self.multimer_type=type 

   def setMultimer(self, ans):
     self.has_multimer=ans 


class Match_struct:

   def __init__(self):

     self.search_results=Write_Search_results.Write_html()
     self.mr_results=Write_MR_results.Write_MR_Results()

     self.OCAmatchfile=''
     self.SSMmatchfile=''
     self.no_of_hits=0
     self.no_of_OCAhits=0
     self.no_of_SSMhits=0
     self.no_of_newSSMhits=0
     self.no_of_PQShits=0
     self.no_of_SCOPhits=0
     self.OCAhits=[]
     self.SSMnewhits=[]
     self.E_value=0.02
     self.est_no_of_mols=0
  
     self.InitError=[]

     self.top_match_chainid=''
     self.top_match_pdbid=''

     self.search_dir=''
     self.models_dir=''
     self.results_dir=''
     self.scratch_dir=''

     self.FastaDB=''
     self.results_html=''
     self.len_html_footer=0
 
     self.chain_list=dict([])
     self.model_list=dict([])
     self.ignore_list=[]
     self.include_list=[]
     self.Domains_list=[]

     self.sorted_list=[]
     self.sorted_MR_list=[]
     self.num_MR_models=0

     self.sorted_model_list=[]
     self.sorted_phaser_models=[]
     self.sorted_molrep_models=[]
     self.sorted_refmac_models=[]

     self.model_counter=0
     self.mr_counter=0
     self.ref_counter=0
     self.solution_found=False
     self.soln_model=''

     self.job_end=False
     self.job_time=0.0
     self.start_time=0.0
     self.end_time=0.0

     self.jobid_array=sets.Set([])
     self.jobid_dict=dict([])
     self.jobname_dict=dict([])

     self.scores_file=''
     self.pickle_file=''
     self.all_raw_file=''
     self.all_align_file=''

     self.MRNUM=0
     self.ENSEMNUM=0
     self.MRPROGRAM=''

     self.SCOP_file=''
     self.SCOP_results_file=''

   def setOCAmatchfile(self, OCA_matchfile):
     self.OCAmatchfile=OCA_matchfile

   def setSSMmatchfile(self, SSM_matchfile):
     self.SSMmatchfile=SSM_matchfile

   def setNoofOCAHits(self, OCAhits):
     self.no_of_OCAhits=OCAhits

   def setNoofSSMHits(self, SSMhits):
     self.no_of_SSMhits=SSMhits

   def setNoofPQSHits(self, no_of_hits):
     self.no_of_PQShits=no_of_hits


   def setTopMatchPDB(self, topmatch_pdb):
     self.top_match_pdbid=topmatch_pdb

   def setTopMatchChain(self, topmatch_chain):
     self.top_match_chainid=topmatch_chain


   def resetJobid_Array(self):
     self.jobid_array=sets.Set([])

   def resetJobid_Dict(self):
     self.jobid_dict=dict([])

   def resetJobName_Dict(self):
     self.jobname_dict=dict([])

   def setSolutionFound(self, solution_found):
      self.solution_found = solution_found

   def setSolutionModel(self, soln_model):
      self.soln_model = soln_model


   def setJob_End(self, flag):
      self.job_end = flag 

   def setJob_Time(self, time):
      self.job_time = time 

   def setStartTime(self, time):
      self.start_time = time 

   def setEndTime(self, time):
      self.end_time = time 


   def setScoresFile(self, filename):
      self.scores_file = filename

   def setPickleFile(self, pickle_file):
      self.pickle_file = pickle_file

   def setAllRawFile(self, filename):
      self.all_raw_file = filename

   def setAllAlignFile(self, filename):
      self.all_align_file = filename


   def setMRNUM(self, MRNUM):
      self.MRNUM = MRNUM

   def setENSEMNUM(self, ENSEMNUM):
      self.ENSEMNUM = ENSEMNUM

   def setMRPROGRAM(self, MRPROGRAM):
      self.MRPROGRAM = MRPROGRAM


   def setE_value(self, E_value):
      self.E_value = E_value

   def setEstNoofMols(self, number):
      self.est_no_of_mols = number


   def setSearchDir(self, directory_name):
      self.search_dir = directory_name

   def setModelsDir(self, directory_name):
      self.models_dir = directory_name

   def setResultsDir(self, directory_name):
      self.results_dir = directory_name

   def setScratchDir(self, directory_name):
      if os.path.isdir(directory_name) == False:
         os.mkdir(directory_name)
      self.scratch_dir = directory_name


   def setFastaDB(self, filename):
      self.FastaDB = filename

   def setResultsHTML(self, filename):
      self.results_html = filename 


   def setSCOPFile(self, filename):
      self.SCOP_file = filename

   def setSCOPResultsFile(self, filename):
      self.SCOP_results_file = filename


class SEQ_match:
   """ A class to retrieve and process matches for the target based in the target sequence. """

   def __init__(self):
      self.CHAIN_NAME = ''
      self.CHAIN_id = ''
      self.PDB_id = ''
      try:
         self.debug=eval(os.environ['MRBUMP_DEBUG'])
      except:
         self.debug=False

   def setDEBUG(self, flag):
      self.debug=flag  

   def get_Fasta_file(self, init, mstat, target_info, OCA_matchfile, MRBUMP_SETUP_DIR):
     """ Top level function for retrieving sequence based matches from the OCA service. """
 
     # Set the e value for the OCA search.
     mstat.setE_value(init.keywords.E_value)
     print "Fasta log message: e value used in Fasta search: %.4f" % mstat.E_value
  
     # Set the fasta DB location
     mstat.setFastaDB(MRBUMP_SETUP_DIR + "/pdb_ATOMseqs.txt")

     # Retrieve the OCA results.
     f=Fasta_search.Fasta()
     if init.keywords.fasta_local:
        print "Fasta log message: Performing Fasta search locally"
        print "Fasta log message: Fasta DB set to: %s" % mstat.FastaDB
        print ""

        # Set the OCA results file (text in local case)
        mstat.setOCAmatchfile(OCA_matchfile + '.txt')
        
        # Perform the search locally
        f.setFastaDB(mstat.FastaDB)
        f.setLogfile(mstat.search_dir + '/logs/fasta_search.log')
        f.fasta_search_local(target_info.seqfile, mstat.OCAmatchfile, mstat.E_value)

     else:
        print "Fasta log message: Performing Fasta search using OCA service"

        # Set the OCA results file (html in web download case)
        mstat.setOCAmatchfile(OCA_matchfile + '.html')
        
        # Perform the search using the OCA service
        f.fasta_search_web(target_info.seqfile, mstat.OCAmatchfile)

   def get_Fasta_matches(self, init, mstat):
      """ A function to retrive the list of PDB and chain IDs from the OCA matches file."""

      # Open the OCA match file for reading 
      match_file=open(mstat.OCAmatchfile,'r')
      line=match_file.readline()
      while line:
         if '>>' in line:
            if init.keywords.fasta_local:
               code=line[2:8].replace(' ','')
            else:
               code=string.split(line,'>')[3].replace('</a','')
               
            # Assign the PDB and chain IDs. If no chain ID set it to 'A'
            self.PDB_id=code[0:4].lower()
            if len(code) == 6:
               self.CHAIN_id=code[5].upper()
            else:
               self.CHAIN_id='A'
          
            self.CHAIN_NAME = self.PDB_id + '_' + self.CHAIN_id

            # Create a new chain object
            c=Chain_struct()
            c.setSource('OCA')
            c.setMultimerType('MONOMER')
            c.setchainName(self.CHAIN_NAME)
            c.setchainID(self.CHAIN_id)
            c.setPDBName(self.PDB_id)
        
            # Create the directory structure for this template structure
            c.setWorkingDIR(mstat.search_dir + '/data/' + c.chainName)

            if self.PDB_id not in mstat.ignore_list:
               # Add this chain object to the list of chains
               mstat.chain_list[self.CHAIN_NAME] = c

               os.mkdir(c.working_DIR)
               os.mkdir(c.working_DIR + '/pdb_file')
               os.mkdir(c.working_DIR + '/pdb_file/unprocessed')
               os.mkdir(c.working_DIR + '/pdbclip')
               os.mkdir(c.working_DIR + '/pdbclip/mr')
               os.mkdir(c.working_DIR + '/pdbclip/refine')
               os.mkdir(c.working_DIR + '/alignments')
             
               # Add this chain name to the list of OCA matches
               mstat.OCAhits.append(self.CHAIN_NAME)
               mstat.no_of_OCAhits = mstat.no_of_OCAhits + 1
               mstat.no_of_hits = mstat.no_of_OCAhits
               if self.debug:
                  print "Added chain: " + self.CHAIN_NAME + " to the match list"
            else:
               print "Fasta log message: Ignoring chain: " + self.CHAIN_NAME + " as it has been excluded from the search"

         line=match_file.readline()

      match_file.close()

      # Add in the included PDB's if they were not already found
      for id in mstat.include_list:
         
         # Convert the id to the correct format before checking it against the list of downloaded chains.
         include_ID=id[0:5].lower() + id[5].upper()
       
         # Check this id against the list of chains found in the searches and add it if not already there.
         if include_ID not in mstat.chain_list.keys():

            chain_id = id[5].upper()
            pdb_id = id[0:4].lower()
            chain = pdb_id + "_" + chain_id

            if self.debug:
               print "Adding chain " + chain + " to the list of search matches."
               print "This chain was not in the fasta results but was selected by user for inclusion."

            c=Chain_struct()
            c.setSource('OCA')
            c.setMultimerType('MONOMER')
            c.setchainName(chain)
            c.setchainID(chain_id)
            c.setPDBName(pdb_id)

            # Create the directory structure for this template structure
            c.setWorkingDIR(mstat.search_dir + '/data/' + c.chainName)

            # Make sure the chain id was not in the ignore list
            if pdb_id not in mstat.ignore_list:
               # Add this chain object to the list of chains
               mstat.chain_list[chain] = c

               os.mkdir(c.working_DIR)
               os.mkdir(c.working_DIR + '/pdb_file')
               os.mkdir(c.working_DIR + '/pdb_file/unprocessed')
               os.mkdir(c.working_DIR + '/pdbclip')
               os.mkdir(c.working_DIR + '/pdbclip/mr')
               os.mkdir(c.working_DIR + '/pdbclip/refine')
               os.mkdir(c.working_DIR + '/alignments')

               # Add this chain name to the list of OCA matches
               mstat.OCAhits.append(chain)
               mstat.no_of_OCAhits = mstat.no_of_OCAhits + 1
               mstat.no_of_hits = mstat.no_of_OCAhits
               if self.debug:
                  print "Added chain: " + chain + " to the match list"
            else:
               sys.stdout.write("Fasta log message: Ignoring chain: " + chain + " as it has been excluded from the search\n")
         else:
            if self.debug:
               print "Chain " + include_ID + " was marked for inclusion as a template chain."
               print "This chain was also found in the FASTA/SSM search."


      sys.stdout.write("\n")

      if mstat.no_of_OCAhits == 0:
        sys.stdout.write("No matches found or all matches have been discounted using Fasta search. Try a higher e-value\n")
        sys.exit(0)

      sys.stdout.write("Fasta log message: Number of valid Fasta matches: %d\n" % mstat.no_of_OCAhits)


class SSM_match:
   """ A class to retrieve and process matches for the target based in the structure of the top hit
       in the OCA search """

   def __init__(self):
      self.CHAIN_NAME = ''
      self.CHAIN_id = ''
      self.PDB_id = ''

      self.perlExecutable='perl'
      self.ssm_script=''
      try:
         self.debug=eval(os.environ['MRBUMP_DEBUG'])
      except:
         self.debug=False

   def setDEBUG(self, flag):
      self.debug=flag  

   def setSSMScript(self, script):
      self.ssm_script=script  

   def get_SSM_file(self, mstat, ehtpx_id, SSM_matchfile):
     """ A function to retrive SSM XML file from the EBI Webservice. """

     # Set the SSM matchfile variable
     mstat.setSSMmatchfile(SSM_matchfile)

     # Call the webservice (replace with python version)
     try:
        os.system(self.perlExecutable + ' ' + self.ssm_script + ' ' + mstat.top_match_pdbid + ' ' + ehtpx_id + ' ' + mstat.SSMmatchfile)
        #os.system("cp /tmp/ssm_result.xml " + SSM_matchfile)
     except:
        print "SSM Error: There was an error when attempting to run the system command:"
        print self.perlExecutable + ' ' + self.ssm_script + ' ' + mstat.top_match_pdbid + ' ' + ehtpx_id + ' ' + mstat.SSMmatchfile
        print "Set SSMSEARCH to 'False' and try again"
        sys.exit(1)
       
   def get_SSM_matches(self, mstat):
      """ Read the SSM xml file and update the chain dictionary with the new (if any) matches. """

      XMLssmfile=open(mstat.SSMmatchfile)
      line=XMLssmfile.readline()

      while line:
        if '<Target>' in line:
           line=XMLssmfile.readline()

           if '<name>' in line:
              self.PDB_id = string.strip(line).replace('<name>','').replace('</name>','')
              line=XMLssmfile.readline()

              self.CHAIN_id = string.strip(line).replace('<chainID>','').replace('</chainID>','')
              if self.CHAIN_id == ' ':
                 self.CHAIN_id = 'A'

              self.CHAIN_NAME = self.PDB_id + '_' + self.CHAIN_id 
              mstat.no_of_SSMhits = mstat.no_of_SSMhits + 1
              mstat.no_of_hits = mstat.no_of_hits + 1

              if mstat.chain_list.has_key(self.CHAIN_NAME) == False:

                 c=Chain_struct()
                 c.setSource('SSM')
                 c.setMultimerType('MONOMER')
                 c.setchainName(self.CHAIN_NAME)
                 c.setchainID(self.CHAIN_id)
                 c.setPDBName(self.PDB_id)

                 # Create the directory structure for this template structure
                 c.setWorkingDIR(mstat.search_dir + '/data/' + c.chainName)

                 # DEV: Remove any chains that are to be ignoreed
                 if self.PDB_id not in mstat.ignore_list:
                    mstat.no_of_newSSMhits = mstat.no_of_newSSMhits + 1

                    os.mkdir(c.working_DIR)
                    os.mkdir(c.working_DIR + '/pdb_file')
                    os.mkdir(c.working_DIR + '/pdb_file/unprocessed')
                    os.mkdir(c.working_DIR + '/pdbclip')
                    os.mkdir(c.working_DIR + '/pdbclip/mr')
                    os.mkdir(c.working_DIR + '/pdbclip/refine')
                    os.mkdir(c.working_DIR + '/alignments')

                    # Add this chain object to the list of chains
                    mstat.chain_list[self.CHAIN_NAME] = c

                    # Add this chain name to the list of new SSM matches
                    mstat.SSMnewhits.append(self.CHAIN_NAME)
                 else:
                    print "SSM log message: Ignoring chain: " + self.CHAIN_NAME + " as it has been excluded from the search"

        else:
           line=XMLssmfile.readline()
      XMLssmfile.close()      

      if mstat.no_of_SSMhits > 0:
         print "SSM log message: Number of hits in SSM search: %s" % mstat.no_of_SSMhits
         print "SSM log message: Number of new hits in SSM search: %s" % mstat.no_of_newSSMhits
      else:
         print "SSM log message: No new hits found in SSM search\n"


class SCOP_match:
   """ A class to perform a search for individual domains using each of the chains collected so far. """

   def __init__(self):
      self.SCOP_temp_URL=''
      try:
         self.debug=eval(os.environ['MRBUMP_DEBUG'])
      except:
         self.debug=False
          
   def setDEBUG(self, flag):
      self.debug=flag  

   def setSCOPtempURL(self, filename):
      self.SCOP_temp_URL=filename
    
   def get_SCOP_matches(self, mstat, MRBUMP_SETUP_DIR):
      """ A function to collect domains from the chains and create new chain list entries. """ 

      # Set the SCOP parsable database file 
      mstat.setSCOPFile(MRBUMP_SETUP_DIR + '/scop_list.txt')

      # Loop over all the chains in the sorted list and process the domain details

      for chain in mstat.sorted_MR_list:
         domains=SCOP_search.Domain_struct()
         domains.findDomains(mstat.SCOP_file, mstat.chain_list[chain].PDBName)

         # Set the number of domains hits for this chain and altogether
         mstat.chain_list[chain].setNoofDomains(domains.no_of_hits)
         mstat.no_of_SCOPhits = mstat.no_of_SCOPhits + domains.no_of_hits

         for i in range(domains.no_of_hits):

           # Add the domains to the domains list for this chain
           mstat.chain_list[chain].domain_list.append(domains.sid[i])

           # If this domain is not in the list already, add it, and setup its variables
           if mstat.chain_list[chain].chainID == (domains.sid[i][5:6]).upper() and \
           os.path.isdir(mstat.search_dir + '/data/' + domains.sid[i]) == False:
              c = Chain_struct()
              c.setWorkingDIR(mstat.search_dir + '/data/' + domains.sid[i])

              # Set up the directory structure for this chain
              os.mkdir(c.working_DIR)
              os.mkdir(c.working_DIR + '/pdb_file')
              os.mkdir(c.working_DIR + '/pdb_file/unprocessed')
              os.mkdir(c.working_DIR + '/pdbclip')
              os.mkdir(c.working_DIR + '/pdbclip/mr')
              os.mkdir(c.working_DIR + '/pdbclip/refine')
              os.mkdir(c.working_DIR + '/alignments')

              # Get the domain ranges
              #domains.getDomainRanges(domains.res_range[i])
              err_code=domains.getDomainCoordinates(domains.sid[i], domains.res_range[i],\
                         mstat.chain_list[chain].PDBFile, c.working_DIR + '/pdb_file')
        
              if err_code == 0:
                 # Reset the starting residue to 0 if it is negative 
                 #if domains.range_list[0][0] < 0:
                 #   domains.range_list[0][0] = 0
   
                 # Extract the domain sequence from the parent chain sequence
                 #domain_sequence = ''
                 #for j in domains.range_list:
                 #   domain_sequence += mstat.chain_list[chain].chain_sequence[j[0]:j[1]]
                 #c.domain_range_list = domains.range_list
   
                 #Set the rest of the domain parameters including the name of the parent chain
                 c.chainID             = (domains.sid[i][5:6]).upper()
                 c.chainName           = domains.sid[i]
                 c.PDBName             = domains.sid[i][0:4]
                 c.unprocessed_PDBFile = mstat.chain_list[chain].unprocessed_PDBFile
                 c.PDBFile             = c.working_DIR + '/pdb_file/' + domains.sid[i] +'.pdb'
                 c.setClipModelPDB(c.PDBFile)
   
                 pdbinfo=PDB_info.PDB_info()
                 c.setChainSequence(pdbinfo.getSequence(c.PDBFile))
                 c.setSeqLength(pdbinfo.sequence_length)
                 c.setNoofRanges(domains.no_of_ranges)
   
                 c.source              = 'SCOP'
                 c.setParentChain(chain)
                  
                 # Add this domain to our list of chains
                 mstat.chain_list[c.chainName]=c
   
                 # Add this domain to the list of domains
                 mstat.Domains_list.append(c.chainName)
   
                 sys.stdout.write("*******************************************************************\n")
                 sys.stdout.write("Name: %s\n" % c.chainName)
                 sys.stdout.write("Seq Length: %d\n" % c.seqlength)
                 sys.stdout.write("PDB file:\n   %s\n" % c.pdbclip_modelPDB)
                 sys.stdout.write("Sequence:\n")
   
                 count=0
                 for i in c.chain_sequence:
                    sys.stdout.write(i)
                    count =  count + 1
                    if count%80==0:
                       sys.stdout.write("\n")
                 if len(c.chain_sequence)%80 != 0:
                    sys.stdout.write("\n")
                     
                 sys.stdout.write("*******************************************************************\n")
                 sys.stdout.write("\n")

              else:
                 # Remove the domain from the domains list for this chain
                 mstat.chain_list[chain].domain_list.remove(domains.sid[i])

class Process_Score:
   """ A class to download and process PDB files and perform Multiple alignment using their sequences.""" 

   def __init__(self):
      try:
         self.debug=eval(os.environ['MRBUMP_DEBUG'])
      except:
         self.debug=False

   def setDEBUG(self, flag):
      self.debug=flag  

   def PDB_process(self, init, mstat):
      """ A function to download and process the template PDB files. """

      PDB=PDB_get()
      
      # Download and process the PDB files for the matches
      PDB.download_PDB(init, mstat)  

      if init.keywords.SCOP:
         print ""
         print "-------------------------"
         print "Preparing SCOP PDB files:"
         print "-------------------------"
         print ""
   
         # PDB files for SCOP matches are derived from the parent chain PDB files
         PDB.download_SCOP_PDB(mstat)  

      print ""
      print "-------------------------------"
      print "Processing PDB files (PDBclip):"
      print "-------------------------------"
      print ""

      # Prepare the pdb files and extract any relevant information from their headers
      PDB.getPDBheader_info(mstat)
      PDB.process_PDB(mstat)
      #PDB.getPDBinfo(mstat)

   def getPDB_sequences(self, mstat, Source):
      """ Get the sequences for each of the Chains in the list."""
       
      if self.debug: 
         print "Extracting sequences from " + mstat.FastaDB +  " for chains in '" + Source + "' search results" 
         print ""

      tmpPDB_file_list=[]
      start_position=0

      gs=Get_sequence_DB.Chain_sequence()
      gs.setPDBseqDB(mstat.FastaDB)

      count=0
      f_DB=open(mstat.FastaDB, "r")
      line=f_DB.readline()
      while line:
         if ">" in line:
            if line[6] == " ":
               chain=line[1:6] + "A"
            else: 
               chain=line[1:7]
            if chain in mstat.chain_list.keys():

               gs.get_seq(chain, start_position)
               start_position=gs.file_position

               # If the sequence was not found in the fasta DB we will attempt to extract it from the PDB file itself
               if gs.sequence == "":
                  if self.debug:
                     print "Sequence for " + chain + " could not be found in " + mstat.FastaDB
                     print "Attempting to extract the sequence from the PDB file...."
                  pg=PDB_get()
               
                  # Assign and note the name of the temporary PDB file
                  tmpPDB_file = mstat.scratch_dir + "/tmpPDB_" + pdb_id + ".pdb"
                  if tmpPDB_file not in tmpPDB_file_list:
                     tmpPDB_file_list.append(tmpPDB_file)
      
                  # Attempt to get the sequence from the PDB atom records
                  sequence=pg.download_PDBsequence(chain, tmpPDB_file)
      
               # Otherwise just set the sequence to be the result from the get_seq routine
               else:
                  sequence=gs.sequence
                  
               mstat.chain_list[chain].setChainSequence(sequence)
               mstat.chain_list[chain].setSeqLength(len(sequence))
      
               if self.debug:
                  sys.stdout.write("Sequence for %s:\n" % chain)
                  res_count=0
                  for x in mstat.chain_list[chain].chain_sequence:
                     sys.stdout.write(x)
                     res_count = res_count + 1
                     if res_count%80 == 0:
                        sys.stdout.write("\n")
                  sys.stdout.write("\n\n")

               count=count+1

         line=f_DB.readline()

      f_DB.close()

      if self.debug: 
         sys.stdout.write("\n")
      else:
         # Remove any temporary PDB files
         for file in tmpPDB_file_list:
            os.remove(file)

   def alignment(self, list, target_info, init, mstat):
      """ A function to do alignment of the target sequence with the sequences from the 
          search results. """
         
      # Write all of the sequences to a fasta file 
      write=Write_seq_align.Write_align_file()
      list=write.write_all_file(list, target_info, mstat)

      # Check to see that we still have some valid matches
      if mstat.no_of_hits == 0:
         sys.stdout.write("Search log message: Number of valid PDB matches found in searches = 0\n")
         sys.stdout.write("Search log message: Exiting program. Try increasing the e-value in the Fasta search.\n")
         sys.stdout.write("\n")
         sys.exit(0)
      else:
         sys.stdout.write("Search log message: Number of valid PDB matches found in searches = %d\n" % mstat.no_of_hits)
         sys.stdout.write("\n")

      if init.keywords.MAPROGRAM == 'MAFFT':
         # Do the multiple alignment
         align=Align_score.Score()
         align.score(list, mstat, mstat.all_raw_file, mstat.all_align_file, 'mafft')

      if init.keywords.MAPROGRAM == 'CLUSTALW':
         # Do the multiple alignment
         align=Align_score.Score()
         align.score(list, mstat, mstat.all_raw_file, mstat.all_align_file, 'clustalw')

      # Set the score and seqID for each chain
      for chain in mstat.chain_list.keys():
         CHAIN = mstat.chain_list[chain]
         CHAIN.setScore(CHAIN.alignment.score)
         CHAIN.setseqID(CHAIN.alignment.seqID)
         CHAIN.setAlignLength(CHAIN.alignment.align_len)
         mstat.chain_list[chain] = CHAIN
  
   def get_top_hit(self, list, mstat):
      """ A function to scan the sequence alignment scores for the hits and pick out the highest """
    
      raw_list=dict([])     
 
      # Set up a dictionary of the scores and chain IDs
      for chain in list:
          
         CHAIN=mstat.chain_list[chain]
         raw_list[CHAIN.chainName] = CHAIN.alignment.score
     
      # Call the sort class to sort the raw dictionary
      s=Sort_dict.Sort_dict()
      mstat.sorted_list=s.sort(raw_list, 'HIGHEST')

      # Assign some parameters related to the top hit
     
      top_hit_chainID = mstat.sorted_list[0][0]
       
      mstat.setTopMatchChain(mstat.chain_list[top_hit_chainID].chainName)
      mstat.setTopMatchPDB(mstat.chain_list[top_hit_chainID].PDBName)

   def prune_list(self, mstat):
      """ A function to reduce the chain list for model preparation and MR. """

      count = 0

      for i in mstat.sorted_list:
         if count < mstat.MRNUM:
            mstat.sorted_MR_list.append(i[0])
         count = count + 1 
      mstat.num_MR_models=len(mstat.sorted_MR_list)

   def print_list(self, mstat):
      """ A function to print out all of the matches from all searches. """ 

      print ""
      print "#####################################################################"
      print "#####              Template Model Search Results                #####" 
      print "#####################################################################"
      print ""
           
      print "CHAIN_ID \t SCORE \t SEQID \t SOURCE"
      print "--------------------------------------------"

      for chain in mstat.sorted_list: 
         if len(chain[0]) == 6:
            # Append a ' ' to the chain ID to get everything to lineup correctly
            chain_id = chain[0] + ' '  
         else:
            chain_id = chain[0]
         print "%s \t %.3f \t %.3f \t %s" % \
            (chain_id, mstat.chain_list[chain[0]].alignment.score, \
                       mstat.chain_list[chain[0]].alignment.seqID, \
                       mstat.chain_list[chain[0]].source)
      print "" 

   def re_sort_MR_list(self, mstat):
      """ A function to create the new sorted list including SCOP results. """

      # Reset the sorted_MR_list to be empty
      mstat.sorted_MR_list = []

      # We want to include all the possible domains of the sorted_MR list in the new list so there 
      # is no pruning of the list.
      for chain in mstat.sorted_list:
         mstat.sorted_MR_list.append(chain[0])
      mstat.num_MR_models=len(mstat.sorted_MR_list)

   def print_pruned_list(self, mstat):
      """ A function to print out the pruned list of search templates (including domains). """ 

      print ""
      print "#####################################################################"
      print "#####      Sorted list of templates to be prepared for MR       #####" 
      print "#####################################################################"
      print ""
           
      print "CHAIN_ID \t SCORE \t SEQID \t SOURCE"
      print "--------------------------------------------"

      for chain in mstat.sorted_MR_list: 
         if len(chain) == 6:
            # Append a ' ' to the chain ID to get everything to lineup correctly
            chain_id = chain + ' '  
         else:
            chain_id = chain
         print "%s \t %.3f \t %.3f \t %s" % \
            (chain_id, mstat.chain_list[chain].alignment.score, \
                       mstat.chain_list[chain].alignment.seqID, \
                       mstat.chain_list[chain].source)
      print "" 


class PDB_get: 
   """ A class to handle downloading and initial processing of the PDB files for each of the 
       template structures. """ 
   
   def __init__(self):
      self.pdbsetEXE       = os.environ['CBIN'] + '/pdbset'
      if os.name == "nt":
         self.coord_formatEXE = os.environ['CBIN'] + '/coord-format'
      else:
         self.coord_formatEXE = os.environ['CBIN'] + '/coord_format'
      self.pdbcurEXE       = os.environ['CBIN'] + '/pdbcur'

      self.download_url1='ftp://ftp.ebi.ac.uk/pub/databases/msd/pdb_uncompressed/'
      self.download_url2='ftp://ftp.rcsb.org/pub/pdb/data/structures/divided/pdb'
      try:
         self.debug=eval(os.environ['MRBUMP_DEBUG'])
      except:
         self.debug=False

   def setDEBUG(self, flag):
      self.debug=flag  

   def download_PDBsequence(self, chain, tmpPDB_file):
      """ A function to download a single PDB. Used as backup for extracting 
          sequences in the getPDB_sequences function. 
          Returns: sequence (string)"""
 
      # Make a note of the chain PDB code and CHAIN identifier
      pdb_id = chain[0:4].lower()
      chain_id = chain[5].upper()
    
      # Some other local variables
      sequence = ''
      pdb_saved = False
      zipped_pdbs = False
      
      # Download the PDB if it is not already saved
      if os.path.isfile(tmpPDB_file) == False:
         try:
            urllib.urlretrieve(self.download_url1 + 'pdb' + pdb_id + '.ent', tmpPDB_file)
            urllib.urlcleanup()
            pdb_saved = True
         except:
            if self.debug:
               sys.stdout.write("Failed to retrieve PDB from first PDB site, trying second site...\n")
               sys.stdout.write("\n")
            try:
                folder_id=pdb_id[1:3].lower()
                urllib.urlretrieve(self.download_url2 + '/' + folder_id + '/pdb' + pdb_id + '.ent.Z', tmpPDB_file + ".Z")
                urllib.urlcleanup()
                if os.name != "nt":
                   os.system("gunzip " +  tmpPDB_file + ".Z")
                else:
                   zipped_pdbs = True
                pdb_saved = True
            except:
               if self.debug == False:
                  print "Sequence retrival error: Error retrieving PDB for %s - could not be found on servers." % pdb_id
               pdb_saved = False
      else:
         pdb_saved = True

      if zipped_pdbs == True and os.name == "nt":
         sys.stdout.write("PDB download log: Can't retieve PDBs at the moment, server is down. Try again later\n")
         sys.stdout.write("\n")
         sys.exit(1)

      # Extract the chain sequence from the PDB coordinates
      if pdb_saved == True:
         info=PDB_info.PDB_info()
         sequence=info.getSequence(tmpPDB_file, chain_id)

      return sequence

   def download_PDB(self, init, mstat):
      """ A function to download the PDB structure files."""
  
      #for chain in mstat.chain_list.keys():
      for chain in mstat.sorted_MR_list:
           
         if mstat.chain_list[chain].source != "SCOP":

            keep_chain = False
            CHAIN = mstat.chain_list[chain] 
         
            # Assign the location of the PDB file for this chain
            CHAIN.setUnprocessedPDBFile(CHAIN.working_DIR + '/pdb_file/unprocessed/' + CHAIN.PDBName + '.pdb')
               
            # Download the PDB structure file for this chain
            if os.path.isfile(CHAIN.unprocessed_PDBFile) == False:
               if init.keywords.ISOLATED == True:
                  if os.path.isfile("/tmp/mrbump_pdb_files/" + CHAIN.PDBName + ".pdb"):
                     shutil.copyfile("/tmp/mrbump_pdb_files/" + CHAIN.PDBName + ".pdb", CHAIN.unprocessed_PDBFile)
                     keep_chain = True
                  else:
                     print "Running in ISOLATED mode. PDB /tmp/mrbump_pdb_files/" + CHAIN.PDBName + ".pdb was not found."
                     print "Ensure that the folder /tmp/mrbump_pdb_files exists and contains all of the PDBs to be used in"
                     print "the current job."
                     sys.exit(1)
               else:
                  try:
                     urllib.urlretrieve(self.download_url1 + '/pdb' + CHAIN.PDBName + '.ent', CHAIN.unprocessed_PDBFile)
                     urllib.urlcleanup()
                     keep_chain = True
                  except:
                     if self.debug:
                         sys.stdout.write("Failed to retrieve PDB from first PDB site, trying second site...\n")
                         sys.stdout.write("\n")
                     try:
                        folder_id=CHAIN.PDBName[1:3].lower()
                        urllib.urlretrieve(self.download_url2 + '/' + folder_id + '/pdb' + CHAIN.PDBName + '.ent.Z', \
                                           CHAIN.unprocessed_PDBFile + ".Z")
                        urllib.urlcleanup()
                        if os.system != "nt":
                           os.system("gunzip " +  CHAIN.unprocessed_PDBFile + ".Z")
                        else:
                           sys.stdout.write("PDB download log: Can't retieve PDBs at the moment, server is down. Try again later\n")
                           sys.stdout.write("\n")
                           sys.exit(1)
                        keep_chain = True
                     except:
                        if self.debug == False:
                           shutil.rmtree(mstat.chain_list[chain].working_DIR)
                        else:
                           print "PDB download log: Error retrieving PDB for %s - could not be found at server" % CHAIN.PDBName
                           print "Removing this entry from the chain list"
                           print ""
                        del mstat.chain_list[chain]
                        keep_chain = False
                        if chain in mstat.sorted_MR_list:
                           print "removing %s from the MR list" % chain 
                           mstat.sorted_MR_list.remove(chain)

	                   # Decrease the number of hits by 1 
                           mstat.num_MR_models = mstat.num_MR_models - 1

               # Re-assign the chain to the stored verion
               if keep_chain == True:
                  if self.debug:
                     print "PDB download log: Downloading: " + CHAIN.PDBName + ".pdb. Saved at: \n " + CHAIN.unprocessed_PDBFile
                     print ""
                  mstat.chain_list[chain] = CHAIN

   def download_SCOP_PDB(self, mstat):
      """ A function to prepare the PDB structure files for the SCOP results."""
  
      for chain in mstat.sorted_MR_list:
         if mstat.chain_list[chain].source == "SCOP":

            keep_chain = False
            CHAIN = mstat.chain_list[chain]
            # Create a keyword line for pdbcur
            pc_key =  'lvres /1/' + CHAIN.chainID + '/' + `CHAIN.domain_range_list[0][0]` + '-' + `CHAIN.domain_range_list[-1][1]` + '\n'
 
            # Create a keyword line for pdbcur to remove the "gap" residues
            if CHAIN.no_of_ranges > 1:
               for i in range(CHAIN.no_of_ranges - 1):
                  pc_key +=  'delres /1/' + CHAIN.chainID + '/' + `CHAIN.domain_range_list[i][1]+1` \
                             + '-' + `CHAIN.domain_range_list[i+1][0]-1` + '\n'
 
            # Finish the key file for pdbcur
            pc_key += 'end'
 
            if self.debug:
               print "SCOP: Pdbcur keywords for " + CHAIN.chainName + ":"
               print ""
               print pc_key
               print "      Output PDB: " + CHAIN.PDBFile
               print ""
 
            # Run pdbcur to extract domain residues
            ### Place choice of paths here ###
            ### This assumes original PDB file in current directory

            if os.path.isfile(CHAIN.unprocessed_PDBFile) == False:
              
               if self.debug:
                  print "SCOP: Unprocessed PDB file not found for chain: " + CHAIN.chainName 
                  print "      Downloading from: " + self.download_url1
 
               CHAIN.setUnprocessedPDBFile(CHAIN.working_DIR + "/pdb_file/unprocessed/" + CHAIN.PDBName + ".pdb")

               try:
                  urllib.urlretrieve(self.download_url1 + 'pdb' + CHAIN.PDBName + '.ent', CHAIN.unprocessed_PDBFile)
                  urllib.urlcleanup()
                  keep_chain = True
               except:
                  if self.debug:
                     sys.stdout.write("Failed to retrieve PDB from first PDB site, trying second site...\n")
                     sys.stdout.write("\n")
                  try:
                     folder_id=CHAIN.PDBName[1:3]
                     urllib.urlretrieve(self.download_url2 + '/' + folder_id + '/pdb' + CHAIN.PDBName + '.ent.Z', CHAIN.unprocessed_PDBFile + ".Z")
                     urllib.urlcleanup()
                     if os.system != "nt":
                        os.system("gunzip " +  CHAIN.unprocessed_PDBFile + ".Z")
                     else:
                        sys.stdout.write("PDB download log: Can't retieve PDBs at the moment, server is down. Try again later\n")
                        sys.stdout.write("\n")
                        sys.exit(1)
                     keep_chain = True
                  except:
                     if self.debug == False:
                        shutil.rmtree(mstat.chain_list[chain].working_DIR)
                     else:
                        print "SCOP: Error retrieving PDB for %s - could not be found at server" % CHAIN.PDBName
                        print "      Removing this entry from the chain list"
                     del mstat.chain_list[chain]
                     keep_chain = False

            termination=False
            if os.path.isfile(CHAIN.unprocessed_PDBFile):
               i,o=os.popen2(self.pdbcurEXE + ' xyzin ' + CHAIN.unprocessed_PDBFile + ' xyzout ' + CHAIN.PDBFile)
 
               # Enter the keywords
               i.write(pc_key)
               i.close()
 
               # Watch the output for successful termination
               out=o.readline()
               pc_log=out
               while out:
                  if 'Normal termination' in out:
                     termination=True

                     # Replace the unprocessed PDB with the PDBcur one to allow for processing later on
                     shutil.copy(CHAIN.PDBFile, CHAIN.unprocessed_PDBFile)
                     os.remove(CHAIN.PDBFile)

                  out=o.readline()
                  pc_log += out
               o.close()
            else:
               print "SCOP PDB creation: Input PDB:\n  " + CHAIN.unprocessed_PDBFile + "\ncould not be found for chain: " + CHAIN.chainName
               pc_log = "Job not started due to missing XYZIN PDB"
 
            if not termination:
               print "SCOP log message: pdbcur failed for %s" % CHAIN.chainName 
               if self.debug:
                  print "Logfile:"
                  print pc_log
               if CHAIN.chainName in mstat.sorted_MR_list:
                  print "removing %s from the MR list" % CHAIN.chainName 
                  mstat.sorted_MR_list.remove(CHAIN.chainName)

          	  # Decrease the number of hits by 1 
                  mstat.num_MR_models = mstat.num_MR_models - 1

 
            # Re-assign the chain to the stored verion
            if keep_chain == True:
               mstat.chain_list[chain] = CHAIN 

 
   def process_PDB(self, mstat):
      """ A function to process the PDB files using pdbset, coord_format and pdbcur. """
 
      print ""
      print "-------------"       
      print "Process PDBs"
      print "-------------"       
      print ""
 
      remove_list=[]
 
      #for chain in mstat.chain_list.keys():
      for chain in mstat.sorted_MR_list:
           
            CHAIN = mstat.chain_list[chain] 
         
            if self.debug:
               print "Processing file: " + CHAIN.PDBName + ".pdb, chain: " + CHAIN.chainID 
               print "-----------------------------------------------"
   
            pdbclip_DIR = CHAIN.working_DIR + '/pdbclip'
   
            CHAIN.setPDBFile(CHAIN.working_DIR + '/pdb_file/' + CHAIN.chainName + '.pdb')
            CHAIN.setClipModelPDB(CHAIN.PDBFile)
   
            # Write the key file for pdbset
            ps_key =  'EXCLUDE HOH\n'
            ps_key += 'END\n'
   
            # Write out the key file if we are running in debug mode
            if self.debug:
               ps_keyfile=pdbclip_DIR + '/ps_key.in'
               ps_key_input=open(ps_keyfile, 'w')
               ps_key_input.write(ps_key)
               ps_key_input.close()
   
            # Run pdbset to remove the waters
            termination=False
            if self.debug:
               print "modifying with pdbset....."
   
            # Set the PDBset logfile
            CHAIN.setPDBsetLog(pdbclip_DIR + '/pdbset_' + CHAIN.chainName + '.log')
   
            # Run pdbset on this pdb file.
            i,o=os.popen2(self.pdbsetEXE + ' xyzin ' + CHAIN.unprocessed_PDBFile + ' xyzout ' + pdbclip_DIR + '/ps_' \
               + CHAIN.chainName + '.pdb')
            i.write(ps_key)
            i.close()

            out=o.readline()
            log=open(CHAIN.pdbset_logfile, "w")
   
            while out:
               if 'Normal completion PDBSET' in out:
                  termination=True
               log.write(out)
               out=o.readline()
            o.close()
            log.close()
   
            if not termination:
               print "PDB download log: pdbset failed for %s" % CHAIN.chainName
	       if CHAIN.chainName not in remove_list:
	          remove_list.append(CHAIN.chainName)
               if self.debug:
                  print "PDB download log: log file can be found at: \n " + CHAIN.pdbset_logfile
   
            # Write the key file for coord_format
            cf_key =  'FIXBLANK\n'
            cf_key += 'END\n'
   
            # Write out the key file if we are running in debug mode
            if self.debug:
               cf_keyfile=pdbclip_DIR + '/cf_key.in'
               cf_key_input=open(cf_keyfile, 'w')
               cf_key_input.write(cf_key)
               cf_key_input.close()
   
            # Run coord_format to fix any chain naming errors
            termination=False
            if self.debug:
               print "modifying with coord_format....."
   
            # Set the PDBset logfile
            CHAIN.setCoordFormatLog(pdbclip_DIR + '/coord_format_' + CHAIN.chainName + '.log')
   
            i,o=os.popen2(self.coord_formatEXE + ' xyzin ' + pdbclip_DIR + '/ps_' + CHAIN.chainName + '.pdb' \
               + ' xyzout ' + pdbclip_DIR + '/cf_' + CHAIN.chainName + '.pdb')
            i.write(cf_key)
            i.close()

            out=o.readline()
            log=open(CHAIN.coord_format_logfile, "w")
   
            while out:
               if 'Normal termination' in out:
                  termination=True
               log.write(out)
               out=o.readline()
            o.close()
            log.close()
   
            if not termination:
               print "PDB download log: coord_format failed for %s" % CHAIN.chainName 
	       if CHAIN.chainName not in remove_list:
	          remove_list.append(CHAIN.chainName)
               if self.debug:
                  print "PDB download log: log file can be found at: \n " + CHAIN.coord_format_logfile

   
            # Write the key file for pdbcur
            pc_key =  'lvchain /1/' + CHAIN.chainID + '\n'
            pc_key += 'delh\n'
            pc_key += 'cutocc\n'
            pc_key += 'mostprob\n'
            pc_key += 'end\n'
   
            # Write out the key file if we are running in debug mode
            if self.debug:
               pc_keyfile=pdbclip_DIR + '/pc_key.in'
               pc_key_input=open(pc_keyfile, 'w')
               pc_key_input.write(pc_key)
               pc_key_input.close()

            # Run pdbcur to strip chains and do some other editing
            termination=False
            if self.debug:
               print "modifying with pdbcur....."
   
            # Set the PDBcur logfile
            CHAIN.setPDBcurLog(pdbclip_DIR + '/pdbcur_' + chain + '.log')
     
            # Run pdbcur on this pdb file
            i,o=os.popen2(self.pdbcurEXE + ' xyzin ' + pdbclip_DIR + '/cf_' + CHAIN.chainName + '.pdb' + ' xyzout ' + CHAIN.PDBFile)
            i.write(pc_key)
            i.close()

            out=o.readline()
            log=open(CHAIN.pdbcur_logfile, "w")
   
            while out:
               if 'Normal termination' in out:
                  termination=True
               log.write(out)
               out=o.readline()
            o.close()
            log.close()
   
            if not termination:
               print "PDB download log: pdbcur failed for %s" % CHAIN.chainName 
	       if CHAIN.chainName not in remove_list:
	          remove_list.append(CHAIN.chainName)
               if self.debug:
                  print "PDB download log: log file can be found at: \n " + CHAIN.pdbcur_logfile
   
            if self.debug:
               print ""
   
            # Test the ATOM records Sequence to see if it is the same as the one we extracted from the
            # parent chain sequence.
            if self.debug and CHAIN.source == "SCOP":
               seq=PDB_info.PDB_info()
               seq.getSequence(CHAIN.PDBFile)
               if seq.ATOMsequence != CHAIN.chain_sequence:
                  print seq.ATOMsequence
                  print CHAIN.chain_sequence
                  print "SCOP: Atom sequence differs from extracted sequence for: " + CHAIN.chainName

            # Re-assign the chain to the stored verion
   
            mstat.chain_list[chain] = CHAIN 

      # Remove the chains from the list that have failed during processing
      for chain in remove_list:
         if self.debug:
            sys.stdout.write("PDB Processing log: Removing chain: " + chain + " as an error was encountered during the processing step\n")
            sys.stdout.write("\n")
         del mstat.chain_list[chain]
         if chain in mstat.sorted_MR_list:
            print "removing %s from the MR list" % chain 
            mstat.sorted_MR_list.remove(chain)

	    # Decrease the number of hits by 1 
            mstat.num_MR_models = mstat.num_MR_models - 1

      # If the number of hits is == 0 then report it and exit
      if mstat.num_MR_models == 0:
         sys.stdout.write("PDB Processing log: Number of valid search models after PDB processing = 0\n") 
         sys.stdout.write("PDB Processing log: Program will exit. Try increasing the number of models to be used in MR\n") 
         sys.stdout.write("\n") 
	 sys.exit(0)
      else:
         sys.stdout.write("PDB Processing log: Number of valid search models after PDB processing = %d\n" % mstat.num_MR_models) 
         sys.stdout.write("\n") 

   def getPDBheader_info(self, mstat):
     """ Extract any relevant header information from the unprocessed PDB files. """
  
     # Loop over all models and find the chain identifier. If not set then set to 'A'
     for chain in mstat.sorted_MR_list:
    
           CHAIN = mstat.chain_list[chain]
    
           # If the PDB exists then extract information for this chain
           if os.path.isfile(CHAIN.unprocessed_PDBFile):
       
              # Call the PDB_info class to get the sequence from the structure file
              info=PDB_info.PDB_info()
   
              # Record the experimental data string
              expdata=info.getEXPDTA(CHAIN.unprocessed_PDBFile)
   
              # Set the CHAIN variables
              CHAIN.setEXPDATA(expdata)
            
              # Check for NMR
              if 'NMR' in CHAIN.expdata:
                 print CHAIN.chainName + " is NMR model" 
              #   if self.debug == False:
              #      shutil.rmtree(CHAIN.working_DIR)
              #   else:
              #      print "PDB Processing log: Chain: " + CHAIN.chainName + \
             #          " removed from search as it is an NMR structure"        
             #    del mstat.chain_list[chain]
             # else:
              mstat.chain_list[chain] = CHAIN
   
           # Otherwise, report an error and remove this model from the chain list
           else:
              if self.debug:
                 print ""
                 print "PDB Processing log: Model " + mstat.chain_list[chain].chainName + \
   	         " - Could not find the unprocessed PDB file for this model."
                 print ""
   
              # Remove it from the list and delete the data directory entry
              if self.debug == False:
                 shutil.rmtree(mstat.chain_list[chain].working_DIR, True)
              del mstat.chain_list[chain] 
              if chain in mstat.sorted_MR_list:
                 print "removing %s from the MR list" % chain 
                 mstat.sorted_MR_list.remove(chain)
   
	         # Decrease the number of hits by 1 
                 mstat.num_MR_models = mstat.num_MR_models - 1

      
   def getPDBinfo(self, mstat):
     """ Extract any relevant information from the PDB files. """
  
     # Loop over all models and find the chain identifier. If not set then set to 'A'
     for chain in mstat.sorted_MR_list:
    
           CHAIN = mstat.chain_list[chain]
    
           # If the PDB exists then extract information for this chain
           if os.path.isfile(CHAIN.PDBFile):
       
              # Call the PDB_info class to get the sequence from the structure file
              info=PDB_info.PDB_info()
   
              # Record the SEQRES and individual chain sequences and the experimental data string
              sequence=info.getSEQRES(CHAIN.PDBFile, CHAIN.chainID)
              chain_sequence=info.getSequence(CHAIN.PDBFile, CHAIN.chainID)
   
              # Set the CHAIN variables
              CHAIN.setSequence(sequence)
              CHAIN.setChainSequence(chain_sequence)
              CHAIN.setSeqLength(int(len(chain_sequence)))
            
              # Test for a ligitemet chain length
              if CHAIN.seqlength <= 5:
                 
                 # if the sequence length is too short, remove that particular chain from the list
                 if self.debug == False:
                    shutil.rmtree(CHAIN.working_DIR)
                 else:
                    print "Removing template " + CHAIN.chainName + " from the list as it has less than 6 Residues"
                 del mstat.chain_list[chain] 
                 if chain in mstat.sorted_MR_list:
                    print "removing %s from the MR list" % chain 
                    mstat.sorted_MR_list.remove(chain)
   
	            # Decrease the number of hits by 1 
                    mstat.num_MR_models = mstat.num_MR_models - 1

              else:
                 mstat.chain_list[chain] = CHAIN
   
           # Otherwise, report an error and remove this model from the chain list
           else:
              if self.debug:
                 print ""
                 print "PDB Processing log: Model " + mstat.chain_list[chain].chainName + \
                  " - Could not find the PDB file for this model."
                 print ""
   
              # Remove it from the list and delete the data directory entry
   
              #shutil.rmtree(mstat.chain_list[chain].working_DIR, True)
              #del mstat.chain_list[chain] 
      
