#! /usr/bin/env python
#
#     Copyright (C) 2005 Ronan Keegan 
#
#     This code is distributed under the terms and conditions of the
#     CCP4 Program Suite Licence Agreement as a CCP4 Application.
#     A copy of the CCP4 licence can be obtained by writing to the
#     CCP4 Secretary, Daresbury Laboratory, Warrington WA4 4AD, UK.
#
#
# Setup scripts for the MR pipeline 
# Ronan Keegan 21/12/04

import os, sys, string
import shutil


class Models_struct:
   """ A class structure to hold model details. """
   
   def __init__(self):
      self.chain_source=''
      self.PDBName=''
      self.seqID=[]

      self.number_ensem=1
      self.number_mols=1
      self.number_domains=1

      self.name=''
      self.type=''
      self.model_directory=''
      self.type_extension='pdb'
      self.multi_type='HOMO'
      self.PDBfile=[]

      self.MR_jobID=0
      self.MR_job_number=0
      self.MR_program=''
      self.Refinement_program=''

      self.phaser_PDBfile=''
      self.phaser_MTZfile=''
      self.phaser_jobname=''
      self.phaser_keyfile=''
      self.phaser_jobID=0
      self.phaser_logfile=''
      self.phaser_solnfile=''
      self.phaser_LLG_score=0.0
      self.phaser_Zscore=0.0
      self.phaser_summary=''

      self.molrep_PDBfile=''
      self.molrep_MTZfile=''
      self.molrep_jobname=''
      self.molrep_keyfile=''
      self.molrep_jobID=0
      self.molrep_logfile=''
      self.molrep_solnfile=''
      self.molrep_Corr=0.0
      self.molrep_Rfac=0.0
      self.molrep_summary=''

      self.refmac_PDBfile=''
      self.refmac_MTZINfile=''
      self.refmac_MTZOUTfile=''
      self.refmac_jobname=''
      self.refmac_keyfile=''
      self.refmac_jobID=0
      self.refmac_logfile=''
      self.refmac_init_score=0.0
      self.refmac_finl_score=0.0
      self.refmac_summary=''

   def setModelChainSource(self, chain_source):
      self.chain_source=chain_source

   def setPDBName(self, PDBName):
      self.PDBName=PDBName

   def setNumberEnsem(self, number_ensem):
     self.number_ensem=number_ensem

   def setNumberMols(self, number_mols):
      self.number_mols=number_mols

   def setNumberDomains(self, number):
      self.number_domains=number

   def setModelName(self, name):
      self.name=name

   def setModelType(self, type):
      self.type=type

   def setModel_directory(self, model_directory):
      self.model_directory=model_directory

   def setModelTypeExtension(self, type_extension):
      self.type_extension=type_extension

   def setMultiType(self, multi_type):
      self.multi_type=multi_type


   def setMR_JobID(self, MR_jobID):
      self.MR_jobID=MR_jobID

   def setMR_JobNumber(self, MR_job_number):
      self.MR_job_number=MR_job_number

   def setMR_Program(self, MR_program):
      self.MR_program=MR_program

   def setRefinement_Program(self, Refinement_program):
      self.Refinement_program=Refinement_program


   def setPhaserPDBfile(self, phaser_pdbfile):
      self.phaser_PDBfile=phaser_pdbfile

   def setPhaserMTZfile(self, phaser_mtzfile):
      self.phaser_MTZfile=phaser_mtzfile

   def setPhaserJobName(self, phaser_jobname):
      self.phaser_jobname=phaser_jobname

   def setPhaserKeyFile(self, phaser_keyfile):
      self.phaser_keyfile=phaser_keyfile

   def setPhaserJobID(self, phaser_jobID):
      self.phaser_jobID=phaser_jobID

   def setPhaserLogFile(self, phaser_logfile):
      self.phaser_logfile=phaser_logfile

   def setPhaserSolnFile(self, phaser_solnfile):
      self.phaser_solnfile=phaser_solnfile

   def setPhaserLLGscore(self, phaser_LLG_score):
      self.phaser_LLG_score=phaser_LLG_score

   def setPhaserZscore(self, phaser_Zscore):
      self.phaser_Zscore=phaser_Zscore

   def setPhaserSummary(self, summary):
      self.phaser_summary=summary



   def setMolrepPDBfile(self, molrep_pdbfile):
      self.molrep_PDBfile=molrep_pdbfile

   def setMolrepMTZfile(self, molrep_mtzfile):
      self.molrep_MTZfile=molrep_mtzfile

   def setMolrepJobName(self, molrep_jobname):
      self.molrep_jobname=molrep_jobname

   def setMolrepKeyFile(self, molrep_keyfile):
      self.molrep_keyfile=molrep_keyfile

   def setMolrepJobID(self, molrep_jobID):
      self.molrep_jobID=molrep_jobID

   def setMolrepLogFile(self, molrep_logfile):
      self.molrep_logfile=molrep_logfile

   def setMolrepSolnFile(self, molrep_solnfile):
      self.molrep_solnfile=molrep_solnfile

   def setMolrepCorr(self, molrep_Corr):
      self.molrep_Corr=molrep_Corr

   def setMolrepRfac(self, molrep_Rfac):
      self.molrep_Rfac=molrep_Rfac

   def setMolrepSummary(self, summary):
      self.molrep_summary=summary


   def setRefmacPDBfile(self, refmac_pdbfile):
      self.refmac_PDBfile=refmac_pdbfile

   def setRefmacMTZINfile(self, refmac_mtzinfile):
      self.refmac_MTZINfile=refmac_mtzinfile

   def setRefmacMTZOUTfile(self, refmac_mtzoutfile):
      self.refmac_MTZOUTfile=refmac_mtzoutfile

   def setRefmacJobName(self, refmac_jobname):
      self.refmac_jobname=refmac_jobname

   def setRefmacKeyFile(self, refmac_keyfile):
      self.refmac_keyfile=refmac_keyfile

   def setRefmacJobID(self, refmac_jobID):
      self.refmac_jobID=refmac_jobID

   def setRefmacLogFile(self, refmac_logfile):
      self.refmac_logfile=refmac_logfile

   def setRefmacInitialScore(self, refmac_init_score):
      self.refmac_init_score=refmac_init_score

   def setRefmacFinalScore(self, refmac_finl_score):
      self.refmac_finl_score=refmac_finl_score

   def setRefmacSummary(self, summary):
      self.refmac_summary=summary


class MR_setup:
   """ A class to setup various structures and lists for running the MR/Refmac pipeline. """
   
   def __init__(self):
      try:
         self.debug=eval(os.environ['MRBUMP_DEBUG'])
      except:
         self.debug=False

   def setDEBUG(self, flag):
      self.debug=flag  

   def setTypes(self, init, mstat, mrsearchdir):
      """ A function to define the model types if the models are not part of an ensemble 
      or a multimer. """
     
      mstat.num_MR_models = 0

      #for chain in mstat.sorted_MODEL_list:
      for chain in mstat.sorted_MR_list:
         for i in init.model_types:
            m=Models_struct()
            m.setModelChainSource(mstat.chain_list[chain].chainName)
            m.setPDBName(mstat.chain_list[chain].PDBName)
            m.seqID.append(mstat.chain_list[chain].seqID)
         
            m.setModelType(i)
            m.setModelName(mstat.chain_list[chain].chainName + '_' + i)

            model_exists=True

            if i == 'PDBCLP':
               m.PDBfile.append(mstat.chain_list[chain].pdbclip_modelPDB)
               m.setModel_directory(mstat.search_dir + '/data/' + mstat.chain_list[chain].chainName + '/pdbclip') 
            elif i == 'MOLREP':
               if os.path.isfile(mstat.chain_list[chain].molrep_modelPDB):
                  m.PDBfile.append(mstat.chain_list[chain].molrep_modelPDB)
                  m.setModel_directory(mstat.search_dir + '/data/' + mstat.chain_list[chain].chainName + '/molrep') 
               else:
                  model_exists=False
            elif i == 'CHNSAW':
               if os.path.isfile(mstat.chain_list[chain].chainsaw_modelPDB):
                  m.PDBfile.append(mstat.chain_list[chain].chainsaw_modelPDB)
                  m.setModel_directory(mstat.search_dir + '/data/' + mstat.chain_list[chain].chainName + '/chainsaw') 
               else:
                  model_exists=False
            elif i == 'PLYALA':
               if os.path.isfile(mstat.chain_list[chain].plyala_modelPDB):
                  m.PDBfile.append(mstat.chain_list[chain].plyala_modelPDB)
                  m.setModel_directory(mstat.search_dir + '/data/' + mstat.chain_list[chain].chainName + '/plyala') 
               else:
                  model_exists=False
  
            # Check that the models were created and add the model to the model_list
            if model_exists == True:
               mstat.model_list[mstat.chain_list[chain].chainName + '_' + i] = m
               mstat.num_MR_models = mstat.num_MR_models + 1

            # Otherwise report an error with the model creation
            else:
               if self.debug:
                  sys.stdout.write("Model preparation log: Excluding %s model for chain %s as its preparation failed\n" % (i, chain)) 

            if self.debug and model_exists == False:
               sys.stdout.write("\n") 

      # Output the list of models if we are running in DEBUG mode
      sys.stdout.write("Model Preparation log: A total of %d models has been created for processing in Molecular Replacement\n" \
                        % mstat.num_MR_models)         
      sys.stdout.write("Model Preparation log: The following list of models has been created for MR:\n")
      sys.stdout.write("\n")
      count=1
      for model in mstat.model_list.keys():
 
          if len(mstat.model_list[model].chain_source) <= 6 and (mstat.model_list[model].chain_source[-1]).isalpha():
             base_type="Chain"
             id=mstat.model_list[model].chain_source[5]
          elif len(mstat.model_list[model].chain_source) == 7:
             base_type="Domain"
             id=mstat.model_list[model].chain_source[5:7]
          elif len(mstat.model_list[model].chain_source) <= 6 and (mstat.model_list[model].chain_source[-1]).isdigit():
             base_type="Multimer"
             id=mstat.model_list[model].chain_source[5]

          sys.stdout.write("%d %s -- " % (count, model))
          if mstat.model_list[model].type == 'PDBCLP':
             sys.stdout.write("%s %s of structure %s prepared using the PDBclip method\n" \
                % (base_type, id, mstat.model_list[model].PDBName)) 
          if mstat.model_list[model].type == 'MOLREP':
             sys.stdout.write("%s %s of structure %s prepared using the Molrep method\n" \
                % (base_type, id, mstat.model_list[model].PDBName)) 
          if mstat.model_list[model].type == 'CHNSAW':
             sys.stdout.write("%s %s of structure %s prepared using the Chainsaw method\n" \
                % (base_type, id, mstat.model_list[model].PDBName)) 
          if mstat.model_list[model].type == 'PLYALA':
             sys.stdout.write("%s %s of structure %s prepared using the Polyalanine method\n" \
                % (base_type, id, mstat.model_list[model].PDBName)) 
          count=count+1
      sys.stdout.write("\n")
      
      # If we are only generating models then create a models directory and copy all of the models to it
      if init.keywords.ONLYMODELS:

         # Create the models directory
         mstat.setModelsDir(mstat.search_dir + '/models')
         os.mkdir(mstat.models_dir)

         sys.stdout.write("These models will be placed in the folder:\n" + mstat.models_dir + "\n")
         sys.stdout.write("\n")

         # Copy all of the generated models to the models directory
         for model in mstat.model_list.keys():
            shutil.copyfile(mstat.model_list[model].PDBfile[0], mstat.models_dir + "/" + model + ".pdb")
